import java.io.File;
import java.io.PrintWriter;

/**
 * Schreibt ein Labyrinth in eine Datei.
 * 
 * @author Bernd Gramlich
 * @version 2013-02-05
 */
public class Datenschreiber
{
    private static Datenschreiber datenschreiberSingleton;

    /*
     * Die Codezeichen, die wir im Labyrinth erwarten.
     */
    private final char WAND      = '#';
    private final char LEERRAUM  = '.';
    private final char EINGANG   = '@';
    private final char SCHATZ    = '$';
    private final char MARKENORD = '^';
    private final char MARKEWEST = '<';
    private final char MARKESUED = 'v';
    private final char MARKEOST  = '>';

    /**
     * Konstruktor.
     * 
     * Der Konstruktor ist private, weil er von aussen
     * nicht aufgerufen werden soll. Stattdessen soll
     * mit gibDatenschreiber () auf den Datenschreiber
     * zugegriffen werden.
     */
    private Datenschreiber ()
    {
    }

    /**
     * Gib den einzigen verfuegbaren Datenschreiber zurueck
     * (also den "datenschreiberSingleton"). Wenn der
     * Datenschreiber noch nicht erzeugt ist, erzeuge ihn.
     */
    public static Datenschreiber gibDatenschreiber ()
    {
        if (null == datenschreiberSingleton)
        {
            datenschreiberSingleton = new Datenschreiber ();
        }

        return datenschreiberSingleton;
    }

    /**
     * Speichere das Labyrinth in die vom Benutzer gewuenschte
     * Labyrinthdatei.
     * 
     * Codierung:
     * WAND ist ein Stueck undurchdringliche Wand.
     * LEERRAUM ist ein Leerraum.
     * EINGANG ist der Ein- und Ausgang.
     * SCHATZ ist ein Schatz.
     * MARKENORD ist eine Marke, die nach Norden zeigt.
     * MARKEWEST  ist eine Marke, die nach Westen zeigt.
     * MARKESUED ist eine Marke, die nach Sueden zeigt.
     * MARKEOST ist eine Marke, die nach Osten zeigt.
     * 
     * Der Rand des Labyrinths wird als Wand interpretiert.
     */
    public boolean schreibe (char[][] labyrinth, File labyrinthDatei)
    {
        try
        {
            /*
             * Oeffne einen Schreiber fuer die Datei.
             */
            PrintWriter schreiber = new PrintWriter (labyrinthDatei);

            /*
             * Drucke das Labyrinth in den Schreiber. Beachte dabei, dass
             * die Ausgabedatei - wie alte Landkarten - nach Osten orientiert
             * ist.
             */
            for (int i = 0; i < labyrinth.length; i++)
            {
                for (int j = 0; j < labyrinth[i].length; j++)
                {
                    char zeichen = labyrinth[i][j];
                    
                    switch (zeichen)
                    {
                        case MARKENORD: zeichen = MARKEOST;  break;
                        case MARKEOST : zeichen = MARKESUED; break;
                        case MARKESUED: zeichen = MARKEWEST; break;
                        case MARKEWEST: zeichen = MARKENORD;  break;
                    }
                    
                    schreiber.print (zeichen);
                }
                
                schreiber.println ();
            }
            
            /*
             * Schliesse den Schreiber.
             */
            schreiber.flush ();
            schreiber.close ();
        }
        catch (Exception fehler)
        {
            return false;
        }
        
        return true;
    }
}
