import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.GridLayout;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingConstants;

/**
 * Ein Dialogfenster fuer das Labyrinthspiel.
 * Vorgesehen sind je ein Button fuer das Vorwaertsgehen,
 * das Drehen nach Links und nach Rechts und das Setzen
 * bzw. Loeschen von Marken, ein Button, um ein neues
 * Labyrinth zu laden, ein Helligkeitsregler und ein Label,
 * in dem (hauptsaechlich) die Anzahl der noch im Labyrinth
 * vorhandenen Schaetze angezeigt wird.
 * 
 * @author Bernd Gramlich 
 * @version 2013-02-05
 */
public class Dialogfenster implements SchatzBeobachter, ActionListener, ChangeListener
{
    /*
     * Das Labyrinthmodell, das mich mit Labyrinthdaten
     * versorgt.
     */
    private LabyrinthModel model;
    
    /*
     * Die Labyrinthsteuerung, die mich verwaltet.
     */
    private LabyrinthController controller;
    
    /*
     * Die Komponenten des Dialogfensters.
     */
    private JFrame dialogfenster;
    private JPanel dialograum;
    private JPanel linksrechtsraum;
    private JButton vorwaertsknopf;
    private JButton linksknopf;
    private JButton rechtsknopf;
    private JButton markenknopf;
    private JSlider lichtregler;
    private JLabel schatzanzeige;
    private JButton ladenknopf;
    private JButton speichernknopf;
    private JButton rueckwaertsknopf;
    
    /*
     * Der Auswahldialog zum Laden eines Labyrinths.
     */
    private JFileChooser dateiauswahl;

    /**
     * Konstruktor.
     * Erzeuge ein Dialogfenster mit dem LabyrinthModel m
     * und dem LabyrinthController c.
     */
    public Dialogfenster (LabyrinthModel m, LabyrinthController c)
    {
        model      = m;
        controller = c;

        model.speichereAlsBeobachter (this);
        
        vorwaertsknopf  = new JButton ("^");
        rueckwaertsknopf = new JButton ("v");
        linksknopf      = new JButton ("<");
        rechtsknopf     = new JButton (">");
        markenknopf     = new JButton ("Marke");
        ladenknopf      = new JButton ("Labyrinth laden");
        speichernknopf  = new JButton ("Labyrinth speichern");
        vorwaertsknopf.addActionListener (this);
        rueckwaertsknopf.addActionListener (this);
        linksknopf.addActionListener (this);
        rechtsknopf.addActionListener (this);
        markenknopf.addActionListener (this);
        ladenknopf.addActionListener (this);
        speichernknopf.addActionListener (this);
        
        schatzanzeige   = new JLabel ("Lade zuerst ein Labyrinth.");
        schatzanzeige.setHorizontalAlignment (SwingConstants.CENTER);
        
        lichtregler     = new JSlider (JSlider.HORIZONTAL, 0, 20, 4);
        lichtregler.addChangeListener (this);
        
        linksrechtsraum = new JPanel (new GridLayout (1, 2));
        linksrechtsraum.add (linksknopf);
        linksrechtsraum.add (rechtsknopf);
        
        dialograum      = new JPanel (new GridLayout (8, 1));
        dialograum.add (vorwaertsknopf);
        dialograum.add (rueckwaertsknopf);
        dialograum.add (linksrechtsraum);
        dialograum.add (markenknopf);
        dialograum.add (lichtregler);
        dialograum.add (schatzanzeige);
        dialograum.add (speichernknopf);
        dialograum.add (ladenknopf);
        
        dialogfenster   = new JFrame ("Labyrinthsteuerung");
        dialogfenster.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
        dialogfenster.getContentPane ().add (dialograum);
        dialogfenster.pack ();
        dialogfenster.setLocationRelativeTo (null);
        dialogfenster.setVisible (true);
        
        dateiauswahl = new JFileChooser ();
    }

    /**
     * Aktualisiere die Anzahl der Schaetze.
     */
    public void aktualisiereSchatzzahl ()
    {
        int schatzzahl = model.gibSchatzzahl ();
        
        if (schatzzahl > 1)
        {
            schatzanzeige.setText (schatzzahl + " Schaetze.");
        }
        else if (1 == schatzzahl)
        {
            schatzanzeige.setText (schatzzahl + " Schatz.");
        }
        else if (0 == schatzzahl)
        {
            schatzanzeige.setText ("Keine Schaetze");
        }
        else
        {
            schatzanzeige.setText ("Spielende.");
            
            controller.ende ();
        }
    }
    
    /**
     * Reagiere auf Benutzeraktionen.
     */
    public void actionPerformed (ActionEvent ereignis)
    {
        if (ereignis.getSource () == vorwaertsknopf)
        {
            controller.gehe ();
        }
        else if (ereignis.getSource () == rueckwaertsknopf)
        {
            controller.geheRueckwaerts ();
        }
        else if (ereignis.getSource () == linksknopf)
        {
            controller.dreheNachLinks ();
        }
        else if (ereignis.getSource () == rechtsknopf)
        {
            controller.dreheNachRechts ();
        }
        else if (ereignis.getSource () == markenknopf)
        {
            controller.markiere ();
        }
        else if (ereignis.getSource () == ladenknopf)
        {
            /*
             * Oeffne das Dateiauswahlfenster zum Einlesen des
             * Labyrinths. Falls der Benutzer eine Datei ausgewaehlt
             * hat ...
             */
            if (dateiauswahl.showOpenDialog (null) == JFileChooser.APPROVE_OPTION)
            { 
                /*
                 * ... lade das Labyrinth von dort.
                 */
                controller.ladeLabyrinth (dateiauswahl.getSelectedFile ());
            }
        }
        
        else if (ereignis.getSource () == speichernknopf)
        {
            /*
             * Oeffne das Dateiauswahlfenster zum Speichern des
             * Labyrinths. Falls der Benutzer eine Datei ausgewaehlt
             * hat ...
             */
            if (dateiauswahl.showSaveDialog (null) == JFileChooser.APPROVE_OPTION)
            { 
                /*
                 * ... speichern das Labyrinth von dort.
                 */
                controller.speichereLabyrinth (dateiauswahl.getSelectedFile ());
            }
        }
    }
    
    /**
     * Reagiere auf Zustandsaenderungen.
     */
    public void stateChanged (ChangeEvent ereignis)
    {
        if (ereignis.getSource () == lichtregler)
        {
            controller.setzeHellsichtweite (lichtregler.getValue (), lichtregler.getMaximum ());
        }
    }
    
    /**
     * Erlaube dem Benutzer, geradeaus zu gehen.
     */
    public void erlaubeGehen (boolean erlaubt)
    {
        vorwaertsknopf.setEnabled (erlaubt);
    }
    
    /**
     * Erlaube dem Benutzer, im Labyrinth rckwrts zu gehen.
     */
    public void erlaubeGeheRueckwaerts (boolean erlaubt)
    {
        rueckwaertsknopf.setEnabled (erlaubt);
    }
    
    /**
     * Erlaube dem Benutzer, sich im Labyrinth zu bewegen.
     */
    public void erlaubeBewegen (boolean erlaubt)
    {
        vorwaertsknopf.setEnabled (erlaubt);
        rueckwaertsknopf.setEnabled (erlaubt);
        linksknopf.setEnabled (erlaubt);
        rechtsknopf.setEnabled (erlaubt);
        markenknopf.setEnabled (erlaubt);
    }
    
    /**
     * Erlaube dem Benutzer, ein Labyrinth zu laden.
     */
    public void erlaubeLaden (boolean erlaubt)
    { 
        ladenknopf.setEnabled (erlaubt);
    }
    
    /**
     * Erlaube dem Benutzer, ein Labyrinth zu speichern.
     */
    public void erlaubeSpeichern (boolean erlaubt)
    { 
        speichernknopf.setEnabled (erlaubt);
    }
    
   
}
