import java.util.Comparator;

/**
 * Eine Vergleichsfunktion fuer Kacheln.
 * Nuetzlich, um Kacheln nach ihrer Entfernung vom Betrachter
 * zu sortieren.
 * 
 * @author Bernd Gramlich 
 * @version 2012-02-05
 */
public class Kachelvergleicher implements Comparator<Kachel>
{
    /**
     * Vergleiche zwei Kacheln hinsichtlich ihres Abstands vom
     * Betrachter.
     * 
     * @param ersteKachel Die erste Kachel.
     * @param zweiteKachel Die zweite Kachel.
     * @return Negativer Wert, falls die ersteKachel naeher am Betrachter
     * ist. Positiver Wert, falls die zweiteKachel naeher am Betrachter ist.
     * 0, falls beide Kacheln gleich weit weg vom Betrachter sind. Bei
     * gleichem Abstand vom Betrachter sollen dabei Marken naeher als Boden-
     * und Deckenkacheln und Boden- und Deckenkacheln naeher als linken
     * rechte und hintere Wandkacheln sein.
     */
    public int compare (Kachel ersteKachel, Kachel zweiteKachel)
    {
        /*
         * Diejenige Kachel ist die kleinere, die den geringeren Abstand vom
         * Betrachter hat.
         */
        int differenz = ersteKachel.gibAbstand () - zweiteKachel.gibAbstand ();
        
        if (0 == differenz)
        {
            /*
             * Bei gleichem Abstand vom Betrachter sind Marken kleiner als
             * Boden- oder Deckenkacheln und Boden- oder Deckenkacheln kleiner
             * als Wandkacheln. (Der Grund ist, dass Wandkacheln Kacheln mit
             * groesserem Abstand und gleichem Blickwinkel zum Betrachter
             * verdecken sollen. Boden- und Deckenkacheln verdecken nichts.
             * Marken werden auf Bodenkacheln gemalt.)
             */
            if (!ersteKachel.istWandkachel ())
            {
                differenz--;
                
                if (ersteKachel.istMarke ())
                {
                    differenz--;
                }
            }
            
            if (!zweiteKachel.istWandkachel ())
            {
                differenz++;
                
                if (zweiteKachel.istMarke ())
                {
                    differenz++;
                }
            }
        }
        
        return differenz;
    }
}
