import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Paint;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.LinkedList;

import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * Leinwand ist eine Klasse, die einfache Zeichenoperationen auf einer
 * leinwandartigen Zeichenflaeche ermoeglicht. Sie ist eine vereinfachte Version
 * der Klasse Canvas (englisch fuer Leinwand) des JDK und wurde speziell fuer das
 * Projekt "Figuren" geschrieben.
 * 
 * 
 * @author: Bruce Quig
 * @author: Michael Koelling (mik)
 * @author: Axel Schmolitzky
 * 
 * 
 * In der erweiterten Fassung kann die Leinwand zusaetzlich auch Texte darstellen
 * und Tastaturbefehle abfragen. Dadurch ist es moeglich, einfache Spiele auf der
 * Leinwand zu programmieren. Die Farbpalette der Methode setzeZeichenfarbe ()
 * wird schrittweise erweitert, um den Anwendungsbeduerfnissen gerecht zu werden.
 * 
 * 
 * @author: Bernd Gramlich
 * 
 * @version: 2013-02-04
 */
public class Leinwand implements ComponentListener {
	// Hinweis: Die Implementierung dieser Klasse (insbesondere die
	// Verwaltung der Farben und Identitaeten der Figuren) ist etwas
	// komplizierter als notwendig. Dies ist absichtlich so, weil damit
	// die Schnittstellen und Exemplarvariablen der Figuren-Klassen
	// fuer den Lernanspruch dieses Projekts einfacher und klarer
	// sein koennen.

	private static Leinwand leinwandSingleton;

	/**
	 * Gib die einzige verfuegbare Leinwand zurueck (also den
	 * "leinwandSingleton"). Wenn die Leinwand noch nicht
	 * erzeugt ist, erzeuge sie mit Defaultwerten fuer Titel,
	 * Breite, Hoehe und Grundfarbe.
	 */
	public static Leinwand gibLeinwand ()
	{
		if (leinwandSingleton == null)
		{
			leinwandSingleton = erzeugeLeinwand ("Leinwand", 800, 600, Color.black);
		}
		leinwandSingleton.setzeSichtbarkeit (true);
		return leinwandSingleton;
	}

	/**
	 * Erzeuge die einzig verfuegbare Leinwand (also den
	 * "leinwandSingleton"). Wenn die Leinwand bereits erzeugt
	 * war, ersetze sie durch eine neue Leinwand.
	 * 
	 * @param titel
	 *            Titel, der im Rahmen der Leinwand angezeigt wird
	 * @param breite
	 *            die gewuenschte Breite der Leinwand
	 * @param hoehe
	 *            die gewuenschte Hoehe der Leinwand
	 * @param grundfarbe
	 *            die Hintergrundfarbe der Leinwand
	 */
	public static Leinwand erzeugeLeinwand (String titel, int breite, int hoehe, Color grundfarbe)
    {
        if (leinwandSingleton != null)
		{
			leinwandSingleton.setzeSichtbarkeit (false);
		}
		leinwandSingleton = new Leinwand (titel, breite, hoehe, grundfarbe);
		leinwandSingleton.setzeSichtbarkeit (true);
		return leinwandSingleton;
	}

	// ----- Exemplarvariablen -----

	private JFrame fenster;

	private Zeichenflaeche zeichenflaeche;

	private Graphics2D graphic;

	private Color hintergrundfarbe;

	private Image leinwandImage;

	private List<Object> figuren;
	
	private List<Object> texte;

	private Map<Object, ShapeMitFarbe> figurZuShape;

	private Map<Object, TextMitFarbe> textZuString;
	
	private LinkedList<KeyEvent> tastenbefehle;
	
	private ArrayList<LeinwandBeobachter> leinwandBeobachter;
	
	/**
	 * Konstruktor.
     * Erzeuge eine Leinwand.
     * 
     * Der Konstruktor ist private, weil er von aussen nicht aufgerufen
     * werden soll. Stattdessen soll mit gibLeinwand () auf die
     * Leinwand zugegriffen werden.
	 * 
	 * @param titel
	 *            Titel, der im Rahmen der Leinwand angezeigt wird
	 * @param breite
	 *            die gewuenschte Breite der Leinwand
	 * @param hoehe
	 *            die gewuenschte Hoehe der Leinwand
	 * @param grundfarbe
	 *            die Hintergrundfarbe der Leinwand
	 */
	private Leinwand(String titel, int breite, int hoehe, Color grundfarbe) {
		fenster = new JFrame();
		zeichenflaeche = new Zeichenflaeche();
		fenster.setContentPane(zeichenflaeche);
		fenster.setTitle(titel);
		fenster.addKeyListener(new Horcher ());
		zeichenflaeche.setPreferredSize(new Dimension(breite, hoehe));
		hintergrundfarbe = grundfarbe;
		fenster.pack();
		figuren = new ArrayList<Object>();
		figurZuShape = new HashMap<Object, ShapeMitFarbe>();
		texte = new ArrayList<Object>();
		textZuString = new HashMap<Object, TextMitFarbe>();
		tastenbefehle = new LinkedList<KeyEvent>();
		leinwandBeobachter = new ArrayList<LeinwandBeobachter>(); 
	}

	/**
	 * Setze, ob diese Leinwand sichtbar sein soll oder nicht. Wenn die Leinwand
	 * sichtbar gemacht wird, wird ihr Fenster in den Vordergrund geholt. Diese
	 * Operation kann auch benutzt werden, um ein bereits sichtbares
	 * Leinwandfenster in den Vordergrund (vor andere Fenster) zu holen.
	 * 
	 * @param sichtbar
	 *            boolean fuer die gewuenschte Sichtbarkeit: true fuer sichtbar,
	 *            false fuer nicht sichtbar.
	 */
	public void setzeSichtbarkeit(boolean sichtbar) {
		if (graphic == null) {
			// erstmaliger Aufruf: erzeuge das Bildschirm-Image und fuelle
			// es mit der Hintergrundfarbe
			Dimension size = zeichenflaeche.getSize();
			leinwandImage = zeichenflaeche.createImage(size.width, size.height);
			graphic = (Graphics2D) leinwandImage.getGraphics();
			graphic.setColor(hintergrundfarbe);
			graphic.fillRect(0, 0, size.width, size.height);
			graphic.setColor(Color.black);
			zeichenflaeche.addComponentListener (this);
		}
		fenster.setVisible(sichtbar);
	}

	/**
	 * Zeichne fuer das gegebene Figur-Objekt eine Java-Figur (einen Shape) auf
	 * die Leinwand.
	 * 
	 * @param figur
	 *            das Figur-Objekt, fuer das ein Shape gezeichnet werden soll
	 * @param farbe
	 *            die Farbe der Figur
	 * @param shape
	 *            ein Objekt der Klasse Shape, das tatsaechlich gezeichnet wird
	 */
	public void zeichne(Object figur, Paint farbe, Shape shape) {
		figuren.remove(figur); // entfernen, falls schon eingetragen
		figuren.add(figur); // am Ende hinzufuegen
		figurZuShape.put(figur, new ShapeMitFarbe(shape, farbe));
	}

	/**
	 * Entferne die gegebene Figur von der Leinwand.
	 * 
	 * @param figur
	 *            die Figur, deren Shape entfernt werden soll
	 */
	public void entferne(Object figur) {
		figuren.remove(figur); // entfernen,falls schon eingetragen
		figurZuShape.remove(figur);
	}

	/**
	 * Schreibe fuer das gegebene Text-Objekt eine Zeichenkette (einen String) auf
	 * die Leinwand.
	 * 
	 * @param text
	 *            das Text-Objekt, fuer das ein String geschrieben werden soll
	 * @param farbe
	 *            die Farbe des Textes
	 * @param zeichenkette
	 *            ein Objekt der Klasse String, das tatsaechlich geschrieben wird
	 * @param x
	 *            x-Koordinate des Textes
	 * @param y
	 *            y-Koordinate des Textes
	 * @param groesse
	 *            Schriftgroesse des Textes
	 */
	public void schreibe(Object text, Color farbe, String zeichenkette, int x, int y, int groesse) {
		texte.remove(text); // entfernen, falls schon eingetragen
		texte.add(text); // am Ende hinzufuegen
		textZuString.put(text, new TextMitFarbe(zeichenkette, farbe, x, y, groesse));
	}

	/**
	 * Radiere den gegebenen Text von der Leinwand.
	 * 
	 * @param text
	 *            der Text, dessen Zeichenkette entfernt werden soll
	 */
	public void radiere(Object text) {
		texte.remove(text); // entfernen,falls schon eingetragen
		textZuString.remove(text);
	}
	/**
	 * Setze die Zeichenfarbe der Leinwand.
	 * 
	 * @param farbe
	 *            die neue Zeichenfarbe.
	 */
	public void setzeZeichenfarbe (Paint farbe)
	{
		graphic.setPaint (farbe);
	}

	/**
	 * Warte fuer die angegebenen Millisekunden. Mit dieser Operation wird eine
	 * Verzoegerung definiert, die fuer animierte Zeichnungen benutzt werden kann.
	 * 
	 * @param millisekunden
	 *            die zu wartenden Millisekunden
	 */
	public void warte(int millisekunden) {
		try {
			Thread.sleep(millisekunden);
		} catch (Exception e) {
			// Exception ignorieren
		}
	}

	/**
	 * Zeichne erneut alle Figuren und Texte auf der Leinwand.
	 */
	public void erneutZeichnen() {
		loeschen();
		for (Object figur : figuren) {
			figurZuShape.get(figur).draw(graphic);
		}
		for (Object text : texte) {
			textZuString.get(text).write(graphic);
		}
		zeichenflaeche.repaint();
	}

	/**
	 * Loesche die gesamte Leinwand.
	 */
	private void loeschen() {
		Color original = graphic.getColor();
		graphic.setColor(hintergrundfarbe);
		Dimension size = zeichenflaeche.getSize();
		graphic.fill(new Rectangle(0, 0, size.width, size.height));
		graphic.setColor(original);
	}
	
	/**
	 * Gib den letzten, noch nicht verarbeiteten Tastendruck zurueck,
	 * den die Leinwand empfangen hat. Falls kein Tastendruck vorkam,
	 * wird null zurueckgegeben.
	 */
	public KeyEvent tastendruck() {
	    warte(5);
	    if (tastenbefehle.size () == 0) {
	        return null;
	    }
	    else {
	        KeyEvent tastenbefehl = tastenbefehle.getFirst();
	        tastenbefehle.removeFirst();
	        return tastenbefehl;
	    }
	}
	
	/**
     * Speichere einen Leinwandbeobachter.
     * 
     * @param b Der Leinwandbeobachter.
     */
    public void speichereAlsBeobachter (LeinwandBeobachter b) {
        leinwandBeobachter.add (b);
    }

    /**
     * Loesche einen Leinwandbeobachter.
     * 
     * @param b Der Leinwandbeobachter.
     */
    public void loescheAlsBeobachter (LeinwandBeobachter b) {
        leinwandBeobachter.remove (b);
    }
    
    /**
     * Nenne die Breite der Zeichenflaeche.
     */
    public int gibBreite () {
        Dimension size = zeichenflaeche.getSize ();
        return size.width;
    }
    
    /**
     * Nenne die Hoehe der Zeichenflaeche.
     */
    public int gibHoehe () {
        Dimension size = zeichenflaeche.getSize ();
        return size.height;
    }
	
	/**
	 * Reagiere nicht auf das Verstecken der Zeichenflaeche.
	 */
	public void componentHidden (ComponentEvent e) {
    }

    /**
	 * Reagiere nicht auf das Bewegen der Zeichenflaeche.
	 */
	public void componentMoved (ComponentEvent e) {
    }

    /**
     * Reagiere auf Groessenaenderungen der Zeichenflaeche, indem Du den
     * Beobachtern die neue Breite und Hoehe mitteilst.
     */
    public void componentResized (ComponentEvent e) {
        Dimension size = zeichenflaeche.getSize();leinwandImage = zeichenflaeche.createImage(size.width, size.height);
        graphic = (Graphics2D) leinwandImage.getGraphics();
        graphic.setColor(hintergrundfarbe);
        graphic.fillRect(0, 0, size.width, size.height);
        graphic.setColor(Color.black);
        
        for (LeinwandBeobachter b: leinwandBeobachter)
        {
            b.aktualisiereLeinwandmasse ();
        }
    }

    /**
	 * Reagiere nicht auf das Zeigen der Zeichenflaeche.
	 */
	public void componentShown (ComponentEvent e) {
    }
	        
	/***************************************************************************
	 * Interne Klasse Zeichenflaeche - die Klasse fuer die GUI-Komponente, die
	 * tatsaechlich im Leinwand-Fenster angezeigt wird. Diese Klasse definiert
	 * ein JPanel mit der zusaetzlichen Moeglichkeit, das auf ihm gezeichnet Image
	 * aufzufrischen (erneut zu zeichnen).
	 */
	private class Zeichenflaeche extends JPanel {
		private static final long serialVersionUID = 20060330L;

		public void paint(Graphics g) {
			g.drawImage(leinwandImage, 0, 0, null);
		}
	}

	/***************************************************************************
	 * Interne Klasse ShapeMitFarbe - Da die Klasse Shape des JDK nicht auch
	 * eine Farbe mitverwalten kann, muss mit dieser Klasse die Verknuepfung
	 * modelliert werden.
	 */
	private class ShapeMitFarbe {
		private Shape shape;

		private Paint farbe;

		public ShapeMitFarbe(Shape shape, Paint farbe) {
			this.shape = shape;
			this.farbe = farbe;
		}

		public void draw(Graphics2D graphic) {
			setzeZeichenfarbe(farbe);
			graphic.fill(shape);
		}
	}

	/***************************************************************************
	 * Interne Klasse TextMitFarbe - Da die Klasse String des JDK nicht auch
	 * eine Farbe mitverwalten kann, muss mit dieser Klasse die Verknuepfung
	 * modelliert werden.
	 */
	private class TextMitFarbe {
		private String zeichenkette;

		private Color farbe;
		
		private int x, y;
		private int groesse;

		public TextMitFarbe(String zeichenkette, Color farbe, int x, int y, int groesse) {
			this.zeichenkette = zeichenkette;
			this.farbe = farbe;
			this.x = x;
			this.y = y;
			this.groesse = groesse;
		}

		public void write(Graphics2D graphic) {
		    Font f = new Font("Serif", 0, groesse);
		    
			setzeZeichenfarbe(farbe);
			graphic.setFont(f);
			graphic.drawString(zeichenkette, x, y);
		}
	}
	
	/****************************************************************************
	 * Interne Klasse Horcher - Speichert alle Tastaturereignisse in der Schlange
	 * tastenbefehle
	 */
	private class Horcher extends KeyAdapter {
        public void keyPressed (KeyEvent taste) {
           tastenbefehle.addLast (taste);
        }
    }
}
