import java.awt.Color;
import java.lang.IllegalArgumentException;

/**
 * Eine Marke auf einer Bodenkachel. Die Marke ist dreieckig
 * mit dem Mittelpunkt als Spitze und zwei benachbarten
 * Eckpunkten als Basisecken. Die genaue Orientierung wird
 * durch zwei Richtungsparameter in der Methode
 * aendereLeinwanddaten () angegeben.
 * 
 * @author Bernd Gramlich 
 * @version 2013-02-05
 */
public class Marke extends Bodenkachel
{
    /**
     * Konstruktor.
     * Erzeuge eine Marke. Die Ausgabeleinwand hat dabei die
     * angegebene Hoehe und Breite und die angegebenen
     * Fluchtpunktkoordinaten.
     * 
     * Wenn man nur den Konstruktor aufruft, wird die Marke
     * als komplett die Bodenkachel bedeckendes Quadrat
     * konstruiert. Das gewuenschte Dreieck erhaelt man erst,
     * wenn man die unten stehende Fassung von
     * aendereLeinwanddaten () mit passenden Richtungsparametern
     * aufruft.
     */
    public Marke (int x, int y, Color dieseFaerbung, int breite, int hoehe, int xf, int yf, double mu, double weite)
        throws IllegalArgumentException
    {
        super (x, y, dieseFaerbung, breite, hoehe, xf, yf, mu, weite);
    }
    
    /**
     * Aendere die Leinwanddaten.
     * Die Marke muss entsprechend den neuen Leinwanddaten geformt
     * und gefaerbt werden.
     * 
     * Dabei bedeuten die Richtungsparameter:
     * xr ==  0 und yr ==  1: Die Marke zeige vorwaerts.
     * xr == -1 und yr ==  0: Die Marke zeige nach links.
     * xr ==  0 und yr == -1: Die Marke zeige rueckwaerts.
     * xr ==  1 und yr ==  0: Die Marke zeige nach rechts.
     */
    public void aendereLeinwanddaten (Color dieseFaerbung, int breite, int hoehe, int xf, int yf, double mu, double weite, int xr, int yr)
    {
        super.aendereLeinwanddaten (dieseFaerbung, breite, hoehe, xf, yf, mu, weite);
        
        if (0 == xr && 1 == yr)
        {
            bild.loescheEcke (3);
            bild.loescheEcke (2);
            bild.ergaenzeEcke (xm, ym);
        }
        else if (-1 == xr && 0 == yr)
        {
           bild.loescheEcke (3);
           bild.loescheEcke (0);
           bild.ergaenzeEcke (xm, ym);
        }
        else if (0 == xr && -1 == yr)
        {
           bild.loescheEcke (1);
           bild.loescheEcke (0);
           bild.ergaenzeEcke (xm, ym);
        }
        else if (1 == xr && 0 == yr)
        {
           bild.loescheEcke (2);
           bild.loescheEcke (1);
           bild.ergaenzeEcke (xm, ym);
        }
    }
    
    /**
     * Pruefe, ob die Kachel eine Bodenmarke ist.
     */
    public boolean istMarke ()
    {
        return true;
    }
}
